program Diagramm_darstellung;
(* ----------------------------------------------------------- *)
(* Funktion:   Darstellung von Diagrammen                      *)
(* Eingabe:    Datei mit maximal 100 Punktkoordinaten der      *)
(*             Form:         x_koordinate, y_koordinate;       *)
(*             Trennzeichen: Komma oder Semikolon              *)
(* Datei:      GRDIAG.PAS                                      *)
(* ----------------------------------------------------------- *)

{$I typedef.bic}              (* Vereinbarungen                *)
{$I graphix.bic}              (* BiC-spezifische Programme     *)
{$I kernel.bic}               (* graf. Grundanweisungen        *)
{$I grdisk.bic}               (* Dateieingabe, -ausgabe        *)
{$I symbol.bic}               (* Punktmarkierungen             *)
{$I axis.hgh}
{$I polygon.hgh}
{$I findwrld.hgh}

var fehler     :boolean;      (* Fehler beim Dateilesen        *)
    fehler_text:wrkstring;    (* Fehlerkommentar               *)
    a:PlotArray;
    dateiname,header:wrkstring;
    anzahl:integer;           (* Punktpaare                    *)
    xmin,xmax,ymin,ymax:real; (* Darstellungsgrenzen           *)
    Line      :integer;       (* Markierung von Punkten        *)
    Lines     :integer;       (* Linienart                     *)
    kurven:integer;           (* Anzahl Kurven                 *)
    datei     :file;
    ant       :string[4];

procedure fehlermeldung(nr:integer);
(* ----------------------------------------------------------- *)
(* Funktion:   Ausgabe einer Fehlermitteilung                  *)
(* ----------------------------------------------------------- *)
begin
fehler:=true;
gotoxy (1,22); write ('FEHLER: ');
case lo(nr) of
  10: write ('Datei ',fehler_text,' nicht vorhanden.');
  11: write ('Zahlendarstellung nicht richtig.');
  12: write ('Weniger als 2 Punkte oder ungerade Anzahl von Werten.');
  13: write ('Mehr als 100 Punkte in Eingabedatei.')
  else write (nr);
end;
writeln (' == ENTER ==');
repeat until keypressed;
gotoxy (1,22); clreol; fehler_text := '';
end;

function grossbuchstaben(strg:wrkstring):wrkstring;
(* ----------------------------------------------------------- *)
(* Funktion:   Umwandlung in Grossbuchstaben                   *)
(* ----------------------------------------------------------- *)
var ausgabe : wrkstring;
    i       : integer;

begin
ausgabe := '';
for i := 1 to length (strg)
    do ausgabe := ausgabe + upcase(copy(strg,i,1));
grossbuchstaben := ausgabe;
end;

procedure einlesen;
(* ---------------------------------------------------------- *)
(* Funktion:   Einlesen von Punktkoordinaten aus Datei        *)
(* ---------------------------------------------------------- *)
var i:integer;
    ziffern:set of char;
    trennzeichen:set of char;
    eingabe:wrkstring;
    zeichen:char;
    kurve:text;

   procedure abspeichern;
   (* ------------------------------------------------------- *)
   (* Funktion:  Abspeichern eines Wertes in Plot-Feld a      *)
   (* ------------------------------------------------------- *)
   var fehler_konv :integer;
       wert        :real;
   begin
   anzahl:=anzahl+1;
   if anzahl>2*MaxPlotGlb
      then fehlermeldung(13)
      else begin
             val(eingabe,wert,fehler_konv);
             if fehler_konv<>0
                then begin
                       fehler_text := '('+eingabe+')';
                       fehlermeldung(11);writeln(eingabe);
                     end
                     else a[anzahl DIV 2 +1,anzahl MOD 2]:=wert;
           end;
   eingabe := '';
   end;

begin
ziffern:=['+','-','.','e','E','0','1','2','3','4','5','6','7','8','9'];
trennzeichen:=[',',';'];

gotoxy (1,22); writeln ('EINGABE: Name der Datei mit Punktkoordinaten ?');
readln(dateiname); dateiname := grossbuchstaben (dateiname);
gotoxy (1,22); clreol; gotoxy (1,23); clreol;
assign (kurve,dateiname);
{$I-} reset(kurve); {$I+};

if ioresult<>0
 then
  begin
   fehler_text := dateiname;
   fehlermeldung(10);
  end
 else
  begin
   anzahl:=0;eingabe:='';
   repeat read(kurve,zeichen);
     if zeichen in trennzeichen
        then abspeichern
        else if zeichen in ziffern
                then eingabe:=eingabe+zeichen;
   until eof(kurve);
   if eingabe <> ''
      then abspeichern;
   if (anzahl MOD 2 <>0) or (anzahl<4)
      then fehlermeldung(12)
      else anzahl:=anzahl DIV 2;
  end;
end;

procedure darstellungsart;
(* ----------------------------------------------------------- *)
(* Funktion:   Eingabe von Punktmarkierung und                 *)
(*             Linienart                                       *)
(* ----------------------------------------------------------- *)
begin
Line:=0;Lines:=0;
gotoxy (1,22); writeln ('EINGABE: Punktmarkierung (0 ... 9) ? ');
readln(Line);  gotoxy (1,22); clreol; gotoxy (1,23); clreol;
if Line<>0
   then Line:=abs(Line) MOD 10;

gotoxy (1,22); writeln ('EINGABE: Linienart  (0 ... 4) ? ');
readln(Lines); gotoxy (1,22); clreol; gotoxy (1,23); clreol;
if Lines<>0
   then Lines:=abs(Lines) MOD 5;
end;

procedure rahmeneingabe;
(* ----------------------------------------------------------- *)
(* Funktion:   Festlegen des Fensters fuer Diagramm            *)
(* ----------------------------------------------------------- *)
var ant:string[4];
    swap:real;

 procedure korrektur(var min,max:real);
 (* ---------------------------------------------------------- *)
 (* Funktion: Sichern von min < max                            *)
 (* ---------------------------------------------------------- *)
 var swap:real;
 begin
 if min=max
    then max:=min+1
     else if min>max
             then begin swap:=min;min:=max;max:=swap;end;
 end;

 procedure windoweingabe;
 (* ---------------------------------------------------------- *)
 (* Funktion:  Eingabe der Darstellungsgrenzen                 *)
 (* ---------------------------------------------------------- *)
 begin
 xmin:=world[1].x1; xmax:=world[1].x2;
 ymin:=world[1].y1; ymax:=world[1].y2;
 gotoxy (1,22); writeln ('EINGABE: X min ? ');readln(xmin);
 gotoxy (1,23); clreol;
 gotoxy (1,22); writeln ('EINGABE: X max ? ');readln(xmax);
 gotoxy (1,23); clreol;
 gotoxy (1,22); writeln ('EINGABE: Y min ? ');readln(ymin);
 gotoxy (1,23); clreol;
 gotoxy (1,22); writeln ('EINGABE: Y max ? ');readln(ymax);
 gotoxy (1,22); clreol; gotoxy (1,23); clreol;
 korrektur(xmin,xmax);korrektur(ymin,ymax);
 DefineWorld(1,xmin,ymax,xmax,ymin);
 end;

 procedure anzeige_darstellungsgrenzen;
 (* ---------------------------------------------------------- *)
 (* Funktion:  Anzeige der Darstellungsgrenzen                 *)
 (* ---------------------------------------------------------- *)
 begin
 gotoxy (1,7);  write ('X min = ',world[1].x1);
 gotoxy (35,7); write ('X max = ',world[1].x2);
 gotoxy (1,8);  write ('Y min = ',world[1].y1);
 gotoxy (35,8); write ('Y max = ',world[1].y2);
 end;

begin
FindWorld(1,a,anzahl,1,1);
with world[1] do
     begin swap:=y1;y1:=y2;y2:=swap;end;
anzeige_darstellungsgrenzen;
gotoxy (1,22);
writeln ('EINGABE: Darstellungsgrenzen aendern (J/N)? ');
readln(ant); gotoxy (1,22); clreol; gotoxy (1,23); clreol;
if ((ant='J') or (ant='j'))
   then windoweingabe;
anzeige_darstellungsgrenzen;
end;

procedure polygon;
(* ----------------------------------------------------------- *)
(* Funktion:   Ausgabe einer Kurve                             *)
(* ----------------------------------------------------------- *)
var LineStyleLoc:integer;
    i:integer;
label ende;

begin
Alpha_scr;
fehler:=false;
einlesen;
if fehler=true
   then goto ende;
kurven:=kurven+1;
if kurven=1
   then begin
           gotoxy (1,22); writeln ('EINGABE: Ueberschrift fuer Diagramm ?');
           readln (header); gotoxy (1,22); clreol; gotoxy (1,23); clreol;
           gotoxy (1,4); writeln (header);
           for i := 1 to length (header)
               do write ('=');

           (* Einstellung der Darstellungsgrenzen bei 1. Kurve *)
           (* Zeichnen der Achsen                              *)
           DefineWindow(1,0,0,XMaxGlb,YMaxGlb);
           Rahmeneingabe;
           SelectWorld(1);
           SelectWindow(1);
           Screen(3,1,1,0);
           DrawAxis(8,8,0,0,0,0,1,1,true);
           Alpha_scr;
           gotoxy (1,2); write ('Grafikbildschirm aktiv');
           gotoxy (1,10); write ('Datei');
           gotoxy (20,10); write ('Punktanzahl');
           gotoxy (35,10); write ('Punktmarkierung');
           gotoxy (55,10); write ('Linienart');
           gotoxy (1,11);
           for i:=1 to 80
               do write ('-');
        end;
if kurven > 9
   then kurven := 9;
gotoxy (1,11+kurven); write (dateiname);
gotoxy (20,11+kurven); write (anzahl);
darstellungsart;
gotoxy (35,11+kurven); write (Line);
gotoxy (55,11+kurven); write (Lines);

Graphic_hr;
LineStyleLoc:=LineStyleGlb;
SetLineStyle(Lines);
AxisGlb:=true;
DrawPolygon(a,1,anzahl,Line,1,0);
SetLineStyle(LineStyleLoc);
repeat until keypressed;
Alpha_scr;
ende:
end;

(* --- Hauptprogramm ---------------------------------------- *)

{$I grdiag.hpt}
